# © 2021 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import models


class ScopAdhesionListeMinistere(models.Model):
    _inherit = "scop.liste.ministere"

    # ------------------------------------------------------
    # Validation de la saisie LM avant chgt statut
    # ------------------------------------------------------
    def check_compulsory_fields_lm(self):
        """
        Check compulsory fields in LM
        :return: errors if exist
        """
        errors = str()
        errors += self.check_fiche_signaletique_filled()
        errors += self.check_director_filled()
        errors += self.check_cap_and_soc_filled()
        errors += self.check_eff_filled()
        errors += self.check_repartition_keys_filled()

        return errors

    def check_fiche_signaletique_filled(self):
        """
        Vérifie que les champs de la fiche signalétique sont bien complétés :
        - objet social
        - date de première signature en coop
        - mois de clôture d'exercice
        """
        error = len(self.social_object) == 0
        error |= self.date_1st_sign is False
        error |= self.closeout_month is False
        if error:
            return (
                "<li>Fiche signalétique : Vérifier que la date de "
                "signature en Coop, l'objet social et le mois de "
                "cloture sont correctement renseignés</li>"
            )
        else:
            return str()

    def check_director_filled(self):
        """
        Vérifie qu'au moins un dirigeant a été renseigné
        """
        error = len(self.director_ids) == 0
        if error:
            return (
                "<li>Membres du CA / Dirigeants : Aucun dirigeant n'a "
                "été renseigné</li>"
            )
        else:
            return str()

    def check_cap_and_soc_filled(self):
        """
        Vérifie que le nombre de parts et la valeur nominale ont
        bien été renseignés
        """
        error = self.cap_tt == 0
        if error:
            return (
                "<li>Capital et Sociétariat : Le calcul [nombre de parts "
                "* valeur nominale] n'a pas été renseigné</li>"
            )
        else:
            return str()

    def check_eff_filled(self):
        """
        Vérifie que les champs de la fiche démographie sont bien renseignés
        si la coop est ni une SCIC, ni une coop47:
        - capital des associés
        - effectif total
        """
        error = False
        if self.partner_id.cooperative_form_id not in (
            self.env.ref("cgscop_partner.form_scic"),
            self.env.ref("cgscop_partner.form_coop47"),
        ):
            error = self.cap_sal_assoc_computed == 0
            error |= self.eff_tt == 0
        if error:
            return (
                "<li>Démographie : Vérifier que tous les champs de "
                'l\'onglet "Démographie" sont bien renseignés '
                "(capital par tranche d'age et effectif total)</li>"
            )
        else:
            return str()

    def check_repartition_keys_filled(self):
        """
        Vérifie que les champs de la fiche démographie sont bien renseignés:
        - capital des associés
        - effectif total
        - équivalent temps plein (pour le calcul de l'adhésion)
        """
        error = False
        if self.partner_id.cooperative_form_id != self.env.ref(
            "cgscop_partner.form_scic"
        ):
            error = (
                self.part_reserve_legale
                + self.part_fond_developpement
                + self.part_capital
                + self.part_travail
                + self.part_autres
                > 100
            )
        if error:
            return (
                "<li>Clés de répartition des bénéfices : "
                "La somme ne peut pas être supérieure à 100</li>"
            )
        else:
            return str()
